"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ElasticsearchFetcher = void 0;
var _shouldIgnoreEsResponseError = require("./shouldIgnoreEsResponseError");
var _nodeUtil = require("node:util");
class ElasticsearchFetcher {
  constructor(params) {
    this.client = params.client;
  }
  async fetch({
    index,
    cursor,
    limit
  }) {
    let response;
    try {
      response = await this.client.search({
        index,
        body: {
          query: {
            match_all: {}
          },
          sort: {
            "id.keyword": {
              order: "asc"
            }
          },
          size: limit + 1,
          track_total_hits: true,
          search_after: cursor,
          _source: false
        }
      });
    } catch (ex) {
      /**
       * If we ignore the error, we can continue with the next index.
       */
      if ((0, _shouldIgnoreEsResponseError.shouldIgnoreEsResponseError)(ex)) {
        if (process.env.DEBUG === "true") {
          console.error((0, _nodeUtil.inspect)(ex, {
            depth: 5,
            showHidden: true
          }));
        }
        return {
          done: true,
          totalCount: 0,
          items: []
        };
      }
      console.error("Failed to fetch data from Elasticsearch.", ex);
      throw ex;
    }
    const {
      hits,
      total
    } = response.body.hits;
    if (hits.length === 0) {
      return {
        done: true,
        cursor: undefined,
        totalCount: total.value,
        items: []
      };
    }
    const hasMoreItems = hits.length > limit;
    let nextCursor;
    if (hasMoreItems) {
      hits.pop();
      nextCursor = hits.at(-1)?.sort;
    }
    const items = hits.reduce((collection, hit) => {
      const [PK, SK] = hit._id.split(":");
      if (!PK || !SK) {
        return collection;
      }
      collection.push({
        PK,
        SK,
        _id: hit._id,
        index: hit._index
      });
      return collection;
    }, []);
    return {
      totalCount: total.value,
      cursor: nextCursor,
      done: !nextCursor,
      items
    };
  }
}
exports.ElasticsearchFetcher = ElasticsearchFetcher;

//# sourceMappingURL=ElasticsearchFetcher.js.map